@@ -7,10 +7,11 @@
 import logging
 import os
 import re
+import sys
 import time
 
 import youtube_dl
-from youtube_dl.utils import DownloadError, sanitize_url
+from youtube_dl.utils import DownloadError, ExtractorError, sanitize_url
 
 import gpodder
 from gpodder import download, feedcore, model, registry, youtube
@@ -67,11 +68,12 @@ class YoutubeCustomDownload(download.CustomDownload):
 
     Actual youtube-dl interaction via gPodderYoutubeDL.
     """
-    def __init__(self, ytdl, url):
+    def __init__(self, ytdl, url, episode):
         self._ytdl = ytdl
         self._url = url
         self._reporthook = None
         self._prev_dl_bytes = 0
+        self._episode = episode
 
     def retrieve_resume(self, tempname, reporthook=None):
         """
@@ -79,6 +81,8 @@ class YoutubeCustomDownload(download.CustomDownload):
         """
         self._reporthook = reporthook
         res = self._ytdl.fetch_video(self._url, tempname, self._my_hook)
+        if 'duration' in res and res['duration']:
+            self._episode.total_time = res['duration']
         headers = {}
         # youtube-dl doesn't return a content-type but an extension
         if 'ext' in res:
@@ -183,7 +187,7 @@ class YoutubeFeed(model.Feed):
         new_entries = [e for e in entries if e['guid'] not in existing_guids]
         logger.debug('%i/%i new entries', len(new_entries), len(all_seen_guids))
         self._ie_result['entries'] = new_entries
-        self._downloader.refresh_entries(self._ie_result, self._max_episodes)
+        self._downloader.refresh_entries(self._ie_result)
         # episodes from entries
         episodes = []
         for en in self._ie_result['entries']:
@@ -256,6 +260,16 @@ class gPodderYoutubeDL(download.CustomDownloader):
             'cachedir': cachedir,
             'no_color': True,  # prevent escape codes in desktop notifications on errors
         }
+        if gpodder.verbose:
+            self._ydl_opts['verbose'] = True
+        else:
+            self._ydl_opts['quiet'] = True
+        # #686 on windows without a console, sys.stdout is None, causing exceptions
+        # when adding podcasts.
+        # See https://docs.python.org/3/library/sys.html#sys.__stderr__ Note
+        if not sys.stdout:
+            logger.debug('no stdout, setting YoutubeDL logger')
+            self._ydl_opts['logger'] = logger
 
     def add_format(self, gpodder_config, opts, fallback=None):
         """ construct youtube-dl -f argument from configured format. """
@@ -272,10 +286,13 @@ class gPodderYoutubeDL(download.CustomDownloader):
         opts['format'] = '/'.join(str(fmt) for fmt in fmt_ids)
         if fallback:
             opts['format'] += '/' + fallback
+        logger.debug('format=%s', opts['format'])
 
     def fetch_video(self, url, tempname, reporthook):
         opts = {
-            'outtmpl': tempname,  # use given tempname by DownloadTask
+            # outtmpl: use given tempname by DownloadTask
+            # (escape % and $ because outtmpl used as a string template by youtube-dl)
+            'outtmpl': tempname.replace('%', '%%').replace('$', '$$'),
             'nopart': True,  # don't append .part (already .partial)
             'retries': 3,  # retry a few times
             'progress_hooks': [reporthook]  # to notify UI
@@ -285,7 +302,7 @@ class gPodderYoutubeDL(download.CustomDownloader):
         with youtube_dl.YoutubeDL(opts) as ydl:
             return ydl.extract_info(url, download=True)
 
-    def refresh_entries(self, ie_result, max_episodes):
+    def refresh_entries(self, ie_result):
         # only interested in video metadata
         opts = {
             'skip_download': True,  # don't download the video
@@ -293,11 +310,22 @@ class gPodderYoutubeDL(download.CustomDownloader):
         }
         self.add_format(self.gpodder_config, opts, fallback='18')
         opts.update(self._ydl_opts)
-        try:
-            with youtube_dl.YoutubeDL(opts) as ydl:
-                ydl.process_ie_result(ie_result, download=False)
-        except DownloadError:
-            logger.exception('refreshing %r', ie_result)
+        new_entries = []
+        # refresh videos one by one to catch single videos blocked by youtube
+        for e in ie_result.get('entries', []):
+            tmp = {k: v for k, v in ie_result.items() if k != 'entries'}
+            tmp['entries'] = [e]
+            try:
+                with youtube_dl.YoutubeDL(opts) as ydl:
+                    ydl.process_ie_result(tmp, download=False)
+                    new_entries.extend(tmp.get('entries'))
+            except DownloadError as ex:
+                if ex.exc_info[0] == ExtractorError:
+                    # for instance "This video contains content from xyz, who has blocked it on copyright grounds"
+                    logger.warning('Skipping %s: %s', e.get('title', ''), ex.exc_info[1])
+                    continue
+                logger.exception('Skipping %r: %s', tmp, ex.exc_info)
+        ie_result['entries'] = new_entries
 
     def refresh(self, url, channel_url, max_episodes):
         """
@@ -363,15 +391,16 @@ class gPodderYoutubeDL(download.CustomDownloader):
         called from registry.custom_downloader.resolve
         """
         if re.match(r'''https://www.youtube.com/watch\?v=.+''', episode.url):
-            return YoutubeCustomDownload(self, episode.url)
+            return YoutubeCustomDownload(self, episode.url, episode)
         elif re.match(r'''https://www.youtube.com/watch\?v=.+''', episode.link):
-            return YoutubeCustomDownload(self, episode.link)
+            return YoutubeCustomDownload(self, episode.link, episode)
         return None
 
 
 class gPodderExtension:
     def __init__(self, container):
         self.container = container
+        self.ytdl = None
 
     def on_load(self):
         self.ytdl = gPodderYoutubeDL(self.container.manager.core.config)
@@ -391,3 +420,16 @@ class gPodderExtension:
             registry.custom_downloader.unregister(self.ytdl.custom_downloader)
         except ValueError:
             pass
+        self.ytdl = None
+
+    def on_ui_object_available(self, name, ui_object):
+        if name == 'gpodder-gtk':
+            self.gpodder = ui_object
+
+    def on_episodes_context_menu(self, episodes):
+        if not self.container.config.manage_downloads \
+                and not all(e.was_downloaded(and_exists=True) for e in episodes):
+            return [(_("Download with Youtube-DL"), self.download_episodes)]
+
+    def download_episodes(self, episodes):
+        self.gpodder.download_episode_list(episodes, downloader=self.ytdl)
